@@ -47,11 +47,17 @@ if IS_INSTALLED:
     if not PYTHONDIR in sys.path:
         sys.path.insert(0, PYTHONDIR)
 else:
-    VERSION =                   "0.3.11"
+    VERSION =                   "0.3.14"
     LOCALE_DIR =                os.path.join(DIRECTORY, "po")
     SHARED_DATA_DIR =           os.path.join(DIRECTORY, "data")
     GLADE_FILE =                os.path.join(DIRECTORY, "data","conduit.glade")
     SHARED_MODULE_DIR =         os.path.join(DIRECTORY, "conduit", "modules")
+                                #{GnomeVfs, GIO, Python}
+    FILE_IMPL =                 os.environ.get("CONDUIT_FILE_IMPL","GnomeVfs")
+                                #{gtkmozembed, webkit, system}
+    BROWSER_IMPL =              os.environ.get("CONDUIT_BROWSER_IMPL","gtkmozembed")
+                                #{GConf,Python}
+    SETTINGS_IMPL =             os.environ.get("CONDUIT_SETTINGS_IMPL","GConf")
 
 import Globals
 GLOBALS = Globals.Globals()
